{ File   : COD2BMP.dpr
  Author : Deniz Oezmen
  Created: 2005-02-17
  Changed: 2005-10-08

  Believe It Or Not: The Riddle of Master Lu COD converter.
}
program COD2BMP;

{$APPTYPE CONSOLE}

uses
  SysUtils, StrUtils, SaveBMP2;

type
  THeader = packed record
              Width,
              Height: Word
            end;

  TBuffer = array of Byte;

var
  CODFile: file;
  Header : THeader;

function Prefix(FileName: string): string;
var
  i: Longint;
begin
  FileName := ExpandFileName(FileName);

  i := Length(FileName) + 1;
  repeat
    Dec(i)
  until (i = 0) or (PosEx('.', FileName, i) > 0);

  if (i = 0) or (PosEx('\', FileName, i + 1) > 0) then
    Result := FileName
  else
    Result := Copy(FileName, 1, i - 1)
end;

function OpenCODFile(var CODFile: file; CODName: string): Boolean;
begin
  FileMode := fmShareDenyWrite;
  AssignFile(CODFile, CODName);

  {$I-}
  Reset(CODFile, 1);
  {$I+}
  OpenCODFile := (IOResult = 0)
end;

function ReadHeader(var Header: THeader): Boolean;
var
  BytesRead: Longint;
begin
  BlockRead(CODFile, Header, SizeOf(Header), BytesRead);
  ReadHeader :=
    (Header.Height * Header.Width + SizeOf(Header) = FileSize(CODFile)) and
    (BytesRead = SizeOf(Header))
end;

procedure CreatePalette(var Palette: TPalette);
var
  i, j: Byte;
begin
  for i := 0 to 255 do
  begin
    with Palette[i] do
    begin
      R := 0;
      for j := 7 downto 1 do
        if i and (1 shl j) > 0 then
          R := 1 shl (7 - j + 1);
      for j := 1 to 7 do
        R := R or (R shr 1);
      G := R;
      B := R
    end
  end
end;

procedure ConvertToBMP(Header: THeader; Palette: TPalette; BMPName: string);
var
  COD: TBuffer;
begin
  SetLength(COD, Header.Width * Header.Height);

  BlockRead(CODFile, COD[0], Header.Width * Header.Height);
  Dump8BitBMP(BMPName, Header.Width, Header.Height, Palette, COD);

  Finalize(COD)
end;

procedure ParseFiles(FilePara: string);
var
  DirPos,
  i        : Byte;
  FileName,
  Dir,
  CODName  : string;
  SearchRec: TSearchRec;
  PFilePara: PChar;
  Palette  : TPalette;
begin
  GetMem(PFilePara, Length(FilePara) + 1);
  StrPCopy(PFilePara, FilePara);

  // get the directory information from FilePara 
  DirPos := 0;
  for i := 1 to Length(FilePara) do
    if FilePara[i] in [':', '\'] then
      DirPos := i;

  Dir := Copy(FilePara, 1, DirPos);

  // search for all files matching FilePara 
  if FindFirst(PFilePara, faAnyFile - faDirectory - faVolumeID,
    SearchRec) = 0 then
  begin
    repeat
      FileName := SearchRec.Name;
      CODName := Dir + FileName;

      // if this file is accessible, analyze it 
      if OpenCODFile(CODFile, CODName) then
        if ReadHeader(Header) then
        begin
          Writeln('Converting ', CODName, ' (', Header.Width, 'x',
            Header.Height, ') ...');
          CreatePalette(Palette);
          ConvertToBMP(Header, Palette, Prefix(CODName) + '.bmp');
          Close(CODFile)
        end
        else
          Writeln(CODName, ' is not a valid COD file.')
      else
        Writeln(CODName, ' not accessible')
    until FindNext(SearchRec) <> 0
  end
  else
    Writeln(FilePara, ' not found')
end;

begin
  if ParamCount = 1 then
    ParseFiles(ParamStr(1))
  else
    Writeln('Incorrect number of parameters.')
end.
