{ File   : DEFExport.pas
  Author : Deniz Oezmen
  Created: 2006-08-13
  Changed: 2006-08-13

  Exports DEF files; tries to emulate the actual formatting used in ROML.
}
unit DEFExport;

interface

uses
  DEFTypes, Classes;

procedure ExportDEF(DEF: TDEF; var Strings: TStringList);

implementation

uses
  SysUtils;

procedure ExportDEF(DEF: TDEF; var Strings: TStringList);
var
  i, j: Longint;
  s   : string;
begin
  Strings.Clear;

  with DEF do
  begin
    // header section
    Strings.Add('scenefile v5_00');
    Strings.Add('');
    Strings.Add(Format('scene: %s', [Scene]));
    Strings.Add('');
    Strings.Add(Format('artbase: "%s"', [Artbase]));
    Strings.Add('');
    Strings.Add(Format('frontscale: %d  backscale: %d',
      [Frontscale, Backscale]));
    Strings.Add(Format('fronty: %d  backy: %d', [Fronty, Backy]));
    Strings.Add('');

    // rails
    if Length(Rails) > 0 then
    begin
      Strings.Add('rails {');
      j := 0;
      s := #009;
      for i := 0 to High(Rails) do
      begin
        s := Format('%s(%d, %d) ', [s, Rails[i].v1, Rails[i].v2]);
        if ((i + 1) mod 4 = 0) or (i = High(Rails)) then
        begin
          Strings.Add(s);
          Inc(j);
          s := #009
        end
      end;
      while j < 8 do
      begin
        Strings.Add(#009);
        Inc(j)
      end;
      Strings.Add('}');
      Strings.Add('')
    end;

    // depths
    s := 'depths (';
    for i := 0 to High(Depths) do
    begin
      s := s + Format('%d', [Depths[i]]);
      if i < High(Depths) then
        s := s + ', '
    end;
    s := s + ')';
    Strings.Add(s);
    Strings.Add('');

    // hotspots
    if Length(Hotspots) > 0 then
    begin
      Strings.Add('hotspots {');
      for i := 0 to High(Hotspots) do
      begin
        with Hotspots[i] do
        begin
          s := Format(#009'{ %d, %d, %d, %d,', [x1, y1, x2, y2]);
          if Sprite <> '' then
            s := Format('%s sprite: "%s",', [s, Sprite]);
          s := Format('%s name: "%s", verb: "%s", prep: "%s",',
            [s, Name, Verb, Prep]);
          Strings.Add(s);
          s := '';
          if Feet_x <> -1 then
            s := Format(#009'  feet_x: %d, feet_y: %d,', [Feet_x, Feet_y]);
          s := Format('%s syntax: %d, facing: %d, cursor: %d }',
            [s, Syntax, Facing, Cursor]);
          Strings.Add(s)
        end
      end;
      Strings.Add('}')
    end  
  end
end;

end.
