{ File   : Main.pas
  Author : Deniz Oezmen
  Created: 2005-03-06
  Changed: 2006-08-13

  Main form control for DEFLoad application.
}
unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls, Menus, StrUtils, FileCtrl;

type
  TMainForm = class(TForm)
    Image: TImage;
    StatusBar: TStatusBar;
    LeftPanel: TPanel;
    LeftBevel: TBevel;
    DrawFacingsCheckBox: TCheckBox;
    DrawRailsCheckBox: TCheckBox;
    DrawHotspotsCheckBox: TCheckBox;
    DrawFeetCheckBox: TCheckBox;
    DEFOpenDialog: TOpenDialog;
    OpenDEFButton: TButton;
    DrawActiveFacingsCheckBox: TCheckBox;
    DrawActiveHotspotsCheckBox: TCheckBox;
    DrawActiveFeetCheckBox: TCheckBox;
    Label1: TLabel;
    Label2: TLabel;
    ExportDEFButton: TButton;
    DEFSaveDialog: TSaveDialog;
    ShowDEFCheckBox: TCheckBox;
    ExportBMPButton: TButton;
    BMPSaveDialog: TSaveDialog;
    procedure ImageMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure OpenDEFButtonClick(Sender: TObject);
    procedure DrawFacingsCheckBoxClick(Sender: TObject);
    procedure DrawRailsCheckBoxClick(Sender: TObject);
    procedure DrawHotspotsCheckBoxClick(Sender: TObject);
    procedure DrawFeetCheckBoxClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ExportDEFButtonClick(Sender: TObject);
    procedure ShowDEFCheckBoxClick(Sender: TObject);
    procedure ExportBMPButtonClick(Sender: TObject);
  private
    procedure DrawHotspot(i: Longint; Active: Boolean);
    procedure DrawHotspots;
    procedure DrawRails;
    procedure DrawFacings;
    procedure DrawFacing(i: Longint; Active: Boolean);
    procedure DrawFeet(i: Longint; Active: Boolean);
    procedure DrawFeets;
    procedure DrawAll;
    procedure ShowDEFContent;
  public
    { Public-Deklarationen }
  end;

var
  MainForm: TMainForm;

implementation

uses
  DEFParser, TTLoader, DEFTypes, DEFExport, DEFContent;

var
  Bitmap : TBitmap;
  Strings: TStringList;
  DEF    : TDEF;

{$R *.dfm}

function CorrectGerman(s: string): string;
begin
  // other characters to replace?
  s := AnsiReplaceStr(s, #001, '');
  s := AnsiReplaceStr(s, #003, '');
  s := AnsiReplaceStr(s, #004, '');
  s := AnsiReplaceStr(s, #005, '');

  Result := s
end;

procedure TMainForm.DrawHotspot(i: Longint; Active: Boolean);
begin
  if i > High(DEF.Hotspots) then
    Exit;

  with DEF.Hotspots[i] do
  begin
    if Active then
      Image.Canvas.Pen.Color := clYellow
    else
      Image.Canvas.Pen.Color := clRed;
    Image.Canvas.Rectangle(x1, y1, x2 + 1, y2 + 1)
  end
end;

procedure TMainForm.DrawHotspots;
var
  i: Longint;
begin
  for i := 0 to High(DEF.Hotspots) do
    DrawHotspot(i, False)
end;

procedure TMainForm.DrawRails;
var
  i: Longint;
begin
  if High(DEF.Rails) > 0 then
  begin
    Image.Canvas.Pen.Color := clWhite;
      with DEF.Rails[0] do
        Image.Canvas.MoveTo(v1, v2)
  end;
  for i := 1 to High(DEF.Rails) do
    with DEF.Rails[i] do
      Image.Canvas.LineTo(v1, v2)
end;

procedure TMainForm.DrawFacing(i: Longint; Active: Boolean);
var
  mx, my: Longint;
begin
  if i > High(DEF.Hotspots) then
    Exit;

  if Active then
    Image.Canvas.Pen.Color := clLime
  else
    Image.Canvas.Pen.Color := clBlue;

  with DEF.Hotspots[i] do
  begin
    mx := x1 + ((x2 - x1) shr 1);
    my := y1 + ((y2 - y1) shr 1);
    Image.Canvas.Rectangle(mx - 1, my - 1, mx + 2, my + 2);
    Image.Canvas.MoveTo(mx, my);
    case Facing of
       0: Image.Canvas.LineTo(mx, my - 10);
       1: Image.Canvas.LineTo(mx + 4, my - 7);
       2: Image.Canvas.LineTo(mx + 7, my - 4);
       3: Image.Canvas.LineTo(mx + 10, my);
       4: Image.Canvas.LineTo(mx + 7, my + 4);
       5: Image.Canvas.LineTo(mx + 4, my + 7);
       6: Image.Canvas.LineTo(mx, my + 10);
       7: Image.Canvas.LineTo(mx - 4, my + 7);
       8: Image.Canvas.LineTo(mx - 7, my + 4);
       9: Image.Canvas.LineTo(mx - 10, my);
      10: Image.Canvas.LineTo(mx - 7, my - 4);
      11: Image.Canvas.LineTo(mx - 4, my - 7)
    end
  end
end;

procedure TMainForm.DrawFacings;
var
  i: Longint;
begin
  for i := 0 to High(DEF.Hotspots) do
    DrawFacing(i, False)
end;

procedure TMainForm.DrawFeet(i: Longint; Active: Boolean);
begin
  if Active then
    Image.Canvas.Pen.Color := clAqua
  else
    Image.Canvas.Pen.Color := clFuchsia;

  with DEF.Hotspots[i] do
  begin
    Image.Canvas.MoveTo(Feet_x - 10, Feet_y - 10);
    Image.Canvas.LineTo(Feet_x + 11, Feet_y + 11);
    Image.Canvas.MoveTo(Feet_x - 10, Feet_y + 10);
    Image.Canvas.LineTo(Feet_x + 11, Feet_y - 11)
  end
end;

procedure TMainForm.DrawFeets;
var
  i: Longint;
begin
  for i := 0 to High(DEF.Hotspots) do
    DrawFeet(i, False)
end;

procedure TMainForm.DrawAll;
begin
  Image.Canvas.Brush.Style := bsClear;
  Image.Canvas.Draw(0, 0, Bitmap);

  image.canvas.pen.color := clWhite;

  if DrawRailsCheckBox.Checked then
    DrawRails;
  if DrawHotspotsCheckBox.Checked then
    DrawHotspots;
  if DrawFacingsCheckBox.Checked then
    DrawFacings;
  if DrawFeetCheckBox.Checked then
    DrawFeets
end;

procedure TMainForm.ImageMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var
  i: Longint;
  s: string;
begin
  DrawAll;

  s := Format('X: %d, Y: %d', [X, Y]);

  for i := 0 to High(DEF.Hotspots) do
  begin
    with DEF.Hotspots[i] do
      if (X >= x1) and (X <= x2) and (Y >= y1) and (Y <= y2) then
      begin
        s := s + ' ' + Name;
        if Prep <> '' then s := s + ' (' + CorrectGerman(Prep) + ')';
        if Verb <> '' then s := s + ' [' + Verb + ']';

        if DrawActiveFeetCheckBox.Checked then
          DrawFeet(i, True);
        if DrawActiveHotspotsCheckBox.Checked then
          DrawHotspot(i, True);
        if DrawActiveFacingsCheckBox.Checked then
          DrawFacing(i, True)
      end
  end;

  StatusBar.SimpleText := s
end;

procedure TMainForm.OpenDEFButtonClick(Sender: TObject);
var
  DEFName,
  TTName : string;
  Result : Longint;
begin
  if not DEFOpenDialog.Execute then
    Exit;

  DEFName := DEFOpenDialog.FileName;

  Result := ParseDEF(DEFName, DEF);

  case Result of
    DEFParser_Success:
      ;
    DEFParser_FileNotOpen:
      begin
        MessageDlg(Format('Could not open %s.', [DEFName]), mtError, [mbOK], 0);
        Exit
      end;
    DEFParser_ParserError:
      begin
        MessageDlg(Format('Error parsing %s.', [DEFName]), mtError, [mbOK], 0);
        Exit
      end
  else
    MessageDlg(Format('ParseDEF returned unknown error code 0x%.8x', [Result]),
      mtError, [mbOK], 0);
    Exit
  end;

  MainForm.Caption := 'DEFLoad - ' + MinimizeName(DEFName, MainForm.Canvas,
    MainForm.Width - 200);

  TTName := ExtractFilePath(DEFName) + DEF.Artbase + '.tt';

  Result := LoadTT(TTName, Bitmap);

  case Result of
    TTLoader_Success:
      ;
    TTLoader_FileNotOpen:
      begin
        MessageDlg(Format('Could not open %s.', [TTName]), mtError, [mbOK], 0);
        Exit
      end;
    TTLoader_InvalidFile:
      begin
        MessageDlg(Format('%s is not a valid TT file.', [TTName]), mtError,
          [mbOK], 0);
        Exit
      end;
    TTLoader_LoadError:
      begin
        MessageDlg(Format('Error loading %s.', [TTName]), mtError, [mbOK], 0);
        Exit
      end
  else
    MessageDlg(Format('LoadTT returned unknown error code 0x%.8x', [Result]),
      mtError, [mbOK], 0);
    Exit
  end;

  MainForm.ClientWidth := Bitmap.Width + 162;
  MainForm.ClientHeight := Bitmap.Height + 19;
  Image.Picture.Bitmap.Width := Bitmap.Width;
  Image.Picture.Bitmap.Height := Bitmap.Height;

  if ShowDEFCheckBox.Checked then
    ShowDEFContent;

  DrawAll
end;

procedure TMainForm.DrawFacingsCheckBoxClick(Sender: TObject);
begin
  DrawAll
end;

procedure TMainForm.DrawRailsCheckBoxClick(Sender: TObject);
begin
  DrawAll
end;

procedure TMainForm.DrawHotspotsCheckBoxClick(Sender: TObject);
begin
  DrawAll
end;

procedure TMainForm.DrawFeetCheckBoxClick(Sender: TObject);
begin
  DrawAll
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  DrawAll
end;

procedure TMainForm.ExportDEFButtonClick(Sender: TObject);
var
  DEFFile: TextFile;
begin
  if not DEFSaveDialog.Execute then
    Exit;

  AssignFile(DEFFile, DEFSaveDialog.FileName);
  {$I-}
  Rewrite(DEFFile);
  {$I+}
  if IOResult <> 0 then
  begin
    MessageDlg(Format('Could not write to file %s.', [DEFSaveDialog.FileName]),
      mtError, [mbOK], 0);
    Exit
  end;

  ExportDEF(DEF, Strings);
  Write(DEFFile, Strings.Text);

  CloseFile(DEFFile)
end;

procedure TMainForm.ShowDEFContent;
begin
  ExportDEF(DEF, Strings);

  DEFShowForm.DEFMemo.Lines.Clear;
  DEFShowForm.DEFMemo.Lines.AddStrings(Strings);

  if not DEFShowForm.Visible then
    DEFShowForm.Show
end;

procedure TMainForm.ShowDEFCheckBoxClick(Sender: TObject);
begin
  if ShowDEFCheckBox.Checked then
    ShowDEFContent
  else if DEFShowForm.Visible then
    DEFShowForm.Close
end;

procedure TMainForm.ExportBMPButtonClick(Sender: TObject);
begin
  if not BMPSaveDialog.Execute then
    Exit;

  Image.Picture.SaveToFile(BMPSaveDialog.FileName);
end;

initialization
  Bitmap := TBitmap.Create;
  Strings := TStringList.Create;

finalization
  Bitmap.Free;
  Strings.Free;
  Finalize(DEF.Rails);
  Finalize(DEF.Depths);
  Finalize(DEF.Hotspots);

end.

