{ File   : ROMLExt3.dpr
  Author : Deniz Oezmen
  Created: 2005-03-03, based on ROMLExtr.cpp created ~2000
  Changed: 2006-03-06

  Believe It Or Not: The Riddle of Master Lu HAS/HAG extractor.
}
program ROMLExt3;

{$APPTYPE CONSOLE}

uses
  SysUtils;

type
  TFileEntry = record
                 FileName : array[0..32] of Char;
                 SectionID: Byte;
                 isEntry  : Boolean;
                 FileOfs,
                 FileSize,
                 EntryOfs : Longint
               end;

  TIDX = record
           FileName: array[0..32] of Char;
           ID      : Byte
         end;

var
  HasFile,
  HagFile,
  DestFile  : file;
  IDXList   : array[0..9] of string;
  NumEntries: Longint;

function OpenHash(HasName: string): Boolean;
begin
  FileMode := fmShareDenyWrite;
  AssignFile(HasFile, HasName);

  {$I-}
  Reset(HasFile, 1);
  {$I+}
  OpenHash := (IOResult = 0)
end;

function AnalyzeHash(HasName: string): Boolean;
var
  i,
  CurFilePos: Longint;
  IDX       : TIDX;
  HagName   : string;
begin
  BlockRead(HasFile, NumEntries, 4);
  Writeln(IntToStr(NumEntries) +
    ' entries (including empty ones) in hash file');
  Writeln('Searching for HAG indexes ...');

  if FileSize(HasFile) > NumEntries * 47 + 4 then
  begin
    Seek(HasFile, NumEntries * 47 + 4);
    CurFilePos := FilePos(HasFile);

		for i := 1 to (FileSize(HasFile) - curFilePos) div SizeOf(TIDX) do
    begin
      BlockRead(HasFile, IDX, Sizeof(TIDX));
      HagName := Copy(IDX.FileName, 1, Pos('.', IDX.FileName)) + 'HAG';
      IDXList[IDX.ID] := hagName;
      Writeln('Found HAG index #' + IntToStr(IDX.ID) + ': ' + IDXList[IDX.ID])
    end;
    Result := True
  end
  else
  begin
    Writeln('Error reading hash file');
    Result := False
  end
end;

procedure CorrectName(var FileName: string);
var
  i: Byte;
begin
  for i := 1 to Length(FileName) do
    if FileName[i] = '/' then
      FileName[i] := '_'
end;

function StrLeadChar(s: string; c: Char; i: Byte): string;
begin
  Result := s;

  while Length(Result) < i do
    Result := c + Result
end;

procedure ExtractFiles(HagDir: string);
var
  i, j,
  RealOfs,
  Files,
  Errors   : Longint;
  FileEntry: TFileEntry;
  HagName,
  FileName,
  NewDir,
  NextFile : string;
  Buffer   : array[1..20480] of Byte;
begin
  Files := 0;
  Errors := 0;

  Writeln(' RealOfs EntryOfs Sct   FileOfs FileSize FileName');
  Seek(HasFile, 0);
  BlockRead(HasFile, NumEntries, 4);

  for i := 1 to NumEntries do
  begin
    RealOfs := FilePos(HasFile);
    BlockRead(HasFile, FileEntry.FileName, 33);
    BlockRead(HasFile, FileEntry.SectionID, 1);
    BlockRead(HasFile, FileEntry.isEntry, 1);
    BlockRead(HasFile, FileEntry.FileOfs, 4);
    BlockRead(HasFile, FileEntry.FileSize, 4);
    BlockRead(HasFile, FileEntry.EntryOfs, 4);
    if FileEntry.isEntry then
    begin
      FileName := FileEntry.FileName;
      CorrectName(FileName);
			NewDir := Copy(IDXList[FileEntry.SectionID], 1,
        Pos('.', IDXList[FileEntry.SectionID]) - 1);

      Writeln(StrLeadChar(IntToStr(RealOfs), ' ', 8), ' ',
        StrLeadChar(IntToStr(FileEntry.EntryOfs), ' ', 8), ' ',
        StrLeadChar(IntToStr(FileEntry.SectionID), ' ', 3), ' ',
        StrLeadChar(IntToStr(FileEntry.FileOfs), ' ', 9), ' ',
        StrLeadChar(IntToStr(FileEntry.FileSize), ' ', 8), ' ',
        FileEntry.FileName);

      HagName := HagDir + '\' + IDXList[FileEntry.SectionID];

      FileMode := fmShareDenyNone;
      AssignFile(HagFile, HagName);
      {$I-}
      Reset(HagFile, 1);
      {$I+}
      if IOResult = 0 then
      begin
        {$I-}
        MkDir(NewDir);
        {$I+}
        IOResult;

        NextFile := NewDir + '\' + FileName;

        FileMode := fmOpenWrite;
        AssignFile(DestFile, NextFile);
        {$I-}
        Rewrite(DestFile, 1);
        {$I+}
        if IOResult = 0 then
        begin
          Seek(HagFile, FileEntry.FileOfs);

          for j := 1 to FileEntry.FileSize div SizeOf(Buffer) do
          begin
            BlockRead(HagFile, Buffer, SizeOf(Buffer));
            BlockWrite(DestFile, Buffer, SizeOf(Buffer))
          end;
          BlockRead(HagFile, Buffer, FileEntry.FileSize mod SizeOf(Buffer));
          BlockWrite(DestFile, Buffer, FileEntry.FileSize mod SizeOf(Buffer));

          CloseFile(DestFile);
        end
        else
          Inc(Errors);
        CloseFile(HagFile)
      end;
      Inc(Files)
    end
    else
    begin
      Writeln(StrLeadChar(IntToStr(RealOfs), ' ', 8), ' ',
        StrLeadChar(IntToStr(FileEntry.EntryOfs), ' ', 8), ' ',
        StrLeadChar(IntToStr(FileEntry.SectionID), ' ', 3), ' ',
        StrLeadChar(IntToStr(FileEntry.FileOfs), ' ', 9), ' ',
        StrLeadChar(IntToStr(FileEntry.FileSize), ' ', 8), ' ',
        '<deleted entry>')
    end
  end;
  Writeln(IntToStr(Files) + ' files extracted, ' + IntToStr(Errors) +
    ' errors occurred')
end;

begin
  Writeln(
    'The Riddle of Master Lu Source file extractor -- (C) Deniz Oezmen, 2000-2005');
  Writeln('early alpha, use at your own risk ...');

  if ParamCount < 2 then
  begin
    Writeln('Please specify hash file and path to HAG-files:');
    Writeln('Usage: ROMLExtr <hash.has> <x:\resource>');
  end
  else
  begin
    if DirectoryExists(ParamStr(2)) and OpenHash(ParamStr(1)) then
      if AnalyzeHash(ParamStr(1)) then
        ExtractFiles(ParamStr(2))
  end
end.
