{ File   : SS2BMP.dpr
  Author : Deniz Oezmen
  Created: 2005-02-15, based on work by Benjamin Haisch
  Changed: 2005-08-25

  Believe It Or Not: The Riddle of Master Lu SS decompressor/extractor.
}
program SS2BMP;

{$APPTYPE CONSOLE}

uses
  SysUtils, StrUtils, SaveBMP2;

type
  THeader       = packed record
                    Magic  : array[0..3] of Char;
                    Unknown: Longint
                  end;
                  (* Magic  : 1295274835 = 'SS4M' [#083#083#052#077]
                     Unknown: always 101 --> "SS for me"/"M4SS"/"MASS"? *)

  TBlockHeader  = packed record
                    BlockID  : array[0..3] of Char;
                    BlockSize: Longint
                  end;
                  (* BlockID  : 542130508 = 'LAP ' [#076#065#080#032]
                                538989395 = 'SS  ' [#083#083#032#032]
                     BlockSize: including Header *)

  TPALHeader    = packed record
                    NumEntries: Longint
                  end;

  TSSHeader     = packed record
                    Unknown1,
                    Unknown2,
                    Unknown3,
                    MaxWidth,
                    MaxHeight,
                    Unknown4,
                    Unknown5,
                    Unknown6,
                    Unknown7,
                    Unknown8,
                    Unknown9,
                    NumFrames: Longint
                  end;

  TSpriteHeader = packed record
                    Unknown1,
                    Unknown2,
                    XPos,     // really?
                    YPos,     // really?
                    Width,
                    Height,
                    Unknown3,
                    Unknown4,
                    Unknown5,
                    Unknown6,
                    Unknown7,
                    Unknown8,
                    Unknown9,
                    Unknown10,
                    Unknown11: Longint
                  end;

  TSpriteEntry  = record
                    Offset: Longint
                  end;

  TSpriteList   = array of TSpriteEntry;

  TBuffer       = array of Byte;

var
  SSFile : file;

function IntToStrL(i: Longint; Len: Byte): string;
begin
  Result := IntToStr(i);

  while Length(Result) < Len do
    Result := '0' + Result
end;

function Prefix(FileName: string): string;
var
  i: Longint;
begin
  FileName := ExpandFileName(FileName);

  i := Length(FileName) + 1;
  repeat
    Dec(i)
  until (i = 0) or (PosEx('.', FileName, i) > 0);

  if (i = 0) or (PosEx('\', FileName, i + 1) > 0) then
    Result := FileName
  else
    Result := Copy(FileName, 1, i - 1)
end;

function OpenSSFile(var SSFile: file; SSName: string): Boolean;
begin
  FileMode := fmShareDenyWrite;
  AssignFile(SSFile, SSName);

  {$I-}
  Reset(SSFile, 1);
  {$I+}
  OpenSSFile := (IOResult = 0)
end;

function ReadHeader: Boolean;
var
  Header   : THeader;
  BytesRead: Longint;
begin
  BlockRead(SSFile, Header, SizeOf(Header), BytesRead);
  ReadHeader := (Header.Magic = 'SS4M') and (BytesRead = SizeOf(Header))
end;

procedure ConvertPalette(var Palette: TPalette);
var
  i: Byte;
begin
  for i := 0 to 255 do
    with Palette[i] do
    begin
      R := R shl 2;
      G := G shl 2;
      B := B shl 2
    end
end;

procedure ReadPalette(var Palette: TPalette);
var
  Buffer   : TPalette;
  PALHeader: TPALHeader;
  i        : Longint;
begin
  Writeln('Reading palette');

  FillChar(Palette, SizeOf(Palette), 0);
  FillChar(Buffer, SizeOf(Palette), 0);

  BlockRead(SSFile, PALHeader, SizeOf(PALHeader));
  BlockRead(SSFile, Buffer, PALHeader.NumEntries * SizeOf(TColor));

  for i := 0 to PALHeader.NumEntries - 1 do
  begin
    Palette[Buffer[i].A].R := Buffer[i].R;
    Palette[Buffer[i].A].G := Buffer[i].G;
    Palette[Buffer[i].A].B := Buffer[i].B
  end;

  ConvertPalette(Palette)
end;

procedure RLEDecode(InBuffer, OutBuffer: TBuffer; InSize, OutSize: Longint);
var
  InPos,
  OutPos,
  i     : Longint;
  b, c  : Byte;
begin
  InPos := 0;
  OutPos := 0;

  while InPos < InSize - 1 do
  begin
    b := InBuffer[InPos];
    Inc(InPos);

    if b = 0 then
    begin
      b := InBuffer[InPos];
      Inc(InPos);
      if b > 0 then
      begin
        for i := 0 to b - 1 do
        begin
          OutBuffer[OutPos] := InBuffer[InPos];
          Inc(InPos);
          if OutPos = OutSize - 1 then
            Break;
          Inc(OutPos)
        end
      end
    end
    else
    begin
      c := InBuffer[InPos];
      Inc(InPos);
      for i := 0 to b - 1 do
      begin
        OutBuffer[OutPos] := c;
        if OutPos = OutSize - 1 then
          Break;
        Inc(OutPos)
      end
    end
  end
end;

procedure ReadSprite(SSName: string; BlockSize: Longint; Palette: TPalette);
var
  SSHeader    : TSSHeader;
  SpriteHeader: TSpriteHeader;
  SpriteList  : TSpriteList;
  BaseOfs,
  InSize,
  OutSize,
  i           : Longint;
  InBuffer,
  OutBuffer   : TBuffer;
  SuffixLen   : Byte;
begin
  BlockRead(SSFile, SSHeader, SizeOf(SSHeader));

  SetLength(SpriteList, SSHeader.NumFrames + 1);
  BlockRead(SSFile, SpriteList[0], SSHeader.NumFrames * SizeOf(TSpriteEntry));

  Writeln('Found ', SSHeader.NumFrames, ' sprite frames');
  SuffixLen := Length(IntToStr(SSHeader.NumFrames));

  BaseOfs := FilePos(SSFile);
  InSize := 0;

  SpriteList[SSHeader.NumFrames].Offset := BlockSize - SizeOf(TBlockHeader) -
    SizeOf(SSHeader) - SizeOf(TSpriteEntry) * SSHeader.NumFrames;

  for i := 0 to SSHeader.NumFrames - 1 do
  begin
    Writeln('  Converting frame #', IntToStrL(i, SuffixLen));

    // really necessary?
    Seek(SSFile, BaseOfs + SpriteList[i].Offset);
    BlockRead(SSFile, SpriteHeader, SizeOf(SpriteHeader));

    with SpriteHeader do
    begin
      // strange: identical offsets, but have to read header ...
      if SpriteList[i + 1].Offset > SpriteList[i].Offset then
        InSize := SpriteList[i + 1].Offset - SpriteList[i].Offset -
          SizeOf(SpriteHeader)
      else
        SpriteList[i + 1].Offset := SpriteList[i].Offset + SizeOf(SpriteHeader);

      OutSize := Width * Height;

      if OutSize > 0 then
      begin
        SetLength(InBuffer, InSize);
        SetLength(OutBuffer, OutSize);

        BlockRead(SSFile, InBuffer[0], InSize);
        RLEDecode(InBuffer, OutBuffer, InSize, OutSize);

        Dump8BitBMP(Format('%s#%.*d.bmp', [Prefix(SSName), SuffixLen, i]),
          Width, Height, Palette, OutBuffer);

        Finalize(InBuffer);
        Finalize(OutBuffer)
      end
      else
        Writeln('    Empty frame')
    end
  end;
  Finalize(SpriteList)
end;

procedure SkipBlock(BlockHeader: TBlockHeader);
begin
  Writeln('Unknown block type ' + BlockHeader.BlockID + ', skipping');
  Seek(SSFile, FilePos(SSFile) + BlockHeader.BlockSize - SizeOf(BlockHeader))
end;

procedure ProcessSS(SSName: string);
var
  BlockHeader: TBlockHeader;
  Palette    : TPalette;
begin
  Writeln('Analyzing ' + SSName);
  while not Eof(SSFile) do
  begin
    BlockRead(SSFile, BlockHeader, SizeOf(BlockHeader));

    if BlockHeader.BlockID = 'LAP ' then
      ReadPalette(Palette)
    else if BlockHeader.BlockID = 'SS  ' then
      ReadSprite(SSName, BlockHeader.BlockSize, Palette)
    else
      SkipBlock(BlockHeader)
  end
end;

procedure ParseFiles(FilePara: string);
var
  DirPos,
  i        : Byte;
  FileName,
  Dir,
  SSName   : string;
  SearchRec: TSearchRec;
  PFilePara: PChar;
begin
  GetMem(PFilePara, Length(FilePara) + 1);
  StrPCopy(PFilePara, FilePara);

  // get the directory information from FilePara
  DirPos := 0;
  for i := 1 to Length(FilePara) do
    if FilePara[i] in [':', '\'] then
      DirPos := i;

  Dir := Copy(FilePara, 1, DirPos);

  // search for all files matching FilePara
  if FindFirst(PFilePara, faAnyFile - faDirectory - faVolumeID,
    SearchRec) = 0 then
  begin
    repeat
      FileName := SearchRec.Name;
      SSName := Dir + FileName;

      (* if this file is accessible, analyze it *)
      if OpenSSFile(SSFile, SSName) then
        if ReadHeader then
        begin
          ProcessSS(SSName);
          Close(SSFile)
        end
        else
          Writeln(SSName, ' is not a valid SS file.')
      else
        Writeln(SSName, ' not accessible')
    until FindNext(SearchRec) <> 0
  end
  else
    Writeln(FilePara, ' not found');
  FreeMem(PFilePara, Length(FilePara) + 1)
end;

begin
  if ParamCount = 1 then
    ParseFiles(ParamStr(1))
  else
    Writeln('Incorrect number of parameters.')
end.
