{ File   : TT2BMP.dpr
  Author : Deniz Oezmen
  Created: 2005-02-09, based on work by Benjamin Haisch
  Changed: 2005-10-08

  Believe It Or Not: The Riddle of Master Lu TT converter.
}
program TT2BMP;

{$APPTYPE CONSOLE}

uses
  SysUtils, StrUtils, SaveBMP2;

type
  THeader = packed record
              Magic,      // 1414799392 = '  TT' [#032#032#084#084]
              DataSize,
              XRes,
              YRes,
              XBlocks,
              YBlocks,
              XBlockSize,
              YBlockSize: Longint;
              Palette   : TPalette
            end;

var
  TTFile : file;
  Header : THeader;

function Prefix(FileName: string): string;
var
  i: Longint;
begin
  FileName := ExpandFileName(FileName);

  i := Length(FileName) + 1;
  repeat
    Dec(i)
  until (i = 0) or (PosEx('.', FileName, i) > 0);

  if (i = 0) or (PosEx('\', FileName, i + 1) > 0) then
    Result := FileName
  else
    Result := Copy(FileName, 1, i - 1)
end;

function OpenTTFile(var TTFile: file; TTName: string): Boolean;
begin
  FileMode := fmShareDenyWrite;
  AssignFile(TTFile, TTName);
  
  {$I-}
  Reset(TTFile, 1);
  {$I+}
  OpenTTFile := (IOResult = 0)
end;

function ReadHeader(var Header: THeader): Boolean;
var
  BytesRead: Longint;
begin
  BlockRead(TTFile, Header, SizeOf(Header), BytesRead);
  ReadHeader := (Header.Magic = 1414799392) and (BytesRead = SizeOf(Header))
end;

procedure ConvertPalette(var Palette: TPalette);
var
  i: Byte;
begin
  for i := 0 to 255 do
    with Palette[i] do
    begin
      R := R shl 2;
      G := G shl 2;
      B := B shl 2
    end
end;

procedure CopyBlock(Block, TT: Pointer; x, y: Longint; Header: THeader);
var
  SourceP,
  DestP,
  XSize,
  YSize,
  i      : Longint;
begin
  if x + Header.XBlockSize <= Header.XRes then
    XSize := Header.XBlockSize
  else
    XSize := Header.XRes mod Header.XBlockSize;
  if y + Header.YBlockSize <= Header.YRes then
    YSize := Header.YBlockSize
  else
    YSize := Header.YRes mod Header.YBlockSize;
  for i := 0 to YSize - 1 do
  begin
    SourceP := i * XSize;
    DestP := (y + i) * Header.XRes + x;
    Move(Pointer(Longint(Block) + SourceP)^,
      Pointer(Longint(TT) + DestP)^,
      XSize)
  end
end;

procedure ConvertToBMP(Header: THeader; BMPName: string);
var
  x, y : Longint;
  TT,
  Block: Pointer;
begin
  GetMem(TT, Header.XRes * Header.YRes);
  GetMem(Block, Header.XBlockSize * Header.YBlockSize);

  for y := 0 to Header.YBlocks - 1 do
    for x := 0 to Header.XBlocks - 1 do
    begin
      BlockRead(TTFile, Block^, Header.XBlockSize * Header.YBlockSize);
      CopyBlock(Block, TT, x * Header.XBlockSize, y * Header.YBlockSize, Header)
    end;

  FreeMem(Block, Header.XBlockSize * Header.YBlockSize);

  Dump8BitBMP(BMPName, Header.XRes, Header.YRes, Header.Palette, TT);

  FreeMem(TT, Header.XRes * Header.YRes)
end;

procedure ParseFiles(FilePara: string);
var
  DirPos,
  i        : Byte;
  FileName,
  Dir,
  TTName   : string;
  SearchRec: TSearchRec;
  PFilePara: PChar;
begin
  GetMem(PFilePara, Length(FilePara) + 1);
  StrPCopy(PFilePara, FilePara);

  // get the directory information from FilePara
  DirPos := 0;
  for i := 1 to Length(FilePara) do
    if FilePara[i] in [':', '\'] then
      DirPos := i;

  Dir := Copy(FilePara, 1, DirPos);

  // search for all files matching FilePara
  if FindFirst(PFilePara, faAnyFile - faDirectory - faVolumeID,
    SearchRec) = 0 then
  begin
    repeat
      FileName := SearchRec.Name;
      TTName := Dir + FileName;

      // if this file is accessible, analyze it
      if OpenTTFile(TTFile, TTName) then
        if ReadHeader(Header) then
        begin
          Writeln('Converting ', TTName, ' (', Header.XRes, 'x',
            Header.YRes, ') ...');
          ConvertPalette(Header.Palette);
          ConvertToBMP(Header, Prefix(TTName) + '.bmp');
          Close(TTFile)
        end
        else
          Writeln(TTName, ' is not a valid TT file.')
      else
        Writeln(TTName, ' not accessible')
    until FindNext(SearchRec) <> 0
  end
  else
    Writeln(FilePara, ' not found')
end;

begin
  if ParamCount = 1 then
    ParseFiles(ParamStr(1))
  else
    Writeln('Incorrect number of parameters.')
end.
